#!/usr/bin/env python
# encoding: utf-8
"""
@author: simin. Gong
@license: (C) Copyright 2013-2023 Spacemit Corporation Limited.
@contact: simin.gong@spacemit.com
@file: app.py
@date: 2024/12/16 19:36 
@desc:
"""
import os
import subprocess
import sys
from functools import wraps
import threading

from flask import Flask, send_from_directory, request, send_file, abort
from flask_cors import CORS
import zipfile
import io

from werkzeug.security import safe_join
from learn_backend.loader import MdLoader
from learn_backend.static.server import dist
from learn_backend.user import User, auth_user
from learn_backend.utils import read_json_file
from learn_backend.version import version

lock = threading.Lock()

dist_path = os.path.dirname(dist.__file__)
home_dir = os.path.expanduser("~")

default_doc_path = os.path.join(home_dir, 'spacemit-riscv-content')
secret_json_file = os.path.join(home_dir, 'secret.json')
need_auth : bool =  True

json_data = read_json_file(secret_json_file)
if json_data is None:
    print('secret file load file!')
    sys.exit(0)
secret_user = User(**json_data)
loader = MdLoader(default_doc_path, False)
loader.load()


app = Flask(__name__, static_folder=dist_path, static_url_path='')
CORS(app)

@app.route('/<path:path>')
def send_vue_app(path):
    response = send_from_directory(app.static_folder, path)
    return response

@app.route('/doc/<path:path>')
def send_vue_doc_app(path):
    response = send_from_directory(app.static_folder, 'index.html')
    return response

@app.route('/auth')
def auth_vue():
    response = send_from_directory(app.static_folder, 'index.html')
    return response

@app.route('/reload')
def reload_vue():
    response = send_from_directory(app.static_folder, 'index.html')
    return response

@app.route('/')
def index():
    response = send_from_directory(app.static_folder, 'index.html')
    return response


@app.after_request
def changeHeader(response):
    disposition = response.get_wsgi_headers('environ').get(
        'Content-Disposition')  or ''
    if disposition.rfind('.js') == len(disposition) - 3:
        response.mimetype = 'application/javascript'
    return response

def token_required(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        if not need_auth:
            return f(*args, **kwargs)
        token = request.headers.get('Authorization')
        if not token or not auth_user(secret_user, token):
            return abort(401)
        return f(*args, **kwargs)

    return decorated


def admin_required(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        if not need_auth:
            return f(*args, **kwargs)
        token = request.headers.get('Authorization')
        if not token or token != secret_user.admin:
            return abort(401)
        return f(*args, **kwargs)

    return decorated


@app.get("/api/md")
@token_required
def get_md_content():
    url = request.args.get('url')
    return loader.get_chapter_md(url)

@app.get("/api/version")
def get_version():
    return version

@app.get("/api/auth")
def auth():
    code = request.args.get('auth')
    return {'auth': auth_user(secret_user, code)}


@app.get("/api/reload")
@token_required
@admin_required
def reload_markdown():
    with lock:
        loader.load()
        return 'ok'

@app.get("/api/update")
@token_required
def update_markdown():
    repo_path = loader.root_path
    try:
        result = subprocess.run(['git', '-C', repo_path, 'pull'], check=True, stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        loader.load()
        return result.stdout.decode()
    except subprocess.CalledProcessError as e:
        print("An error occurred while updating the repository:", e.stderr.decode())
        return e.stderr.decode()


@app.route('/files/<path:subpath>', methods=['GET'])
def download_files(subpath):
    code = request.args.get('token')
    if need_auth and not auth_user(secret_user, code):
        abort(404)
    safe_path = safe_join(loader.doc_path, subpath)
    if not os.path.exists(safe_path):
        abort(404)
    if os.path.isdir(safe_path):
        dir_name = os.path.basename(safe_path)
        zip_buffer = io.BytesIO()
        with zipfile.ZipFile(zip_buffer, 'w', zipfile.ZIP_DEFLATED) as zip_file:
            for root, _, files in os.walk(safe_path):
                for file in files:
                    file_path = os.path.join(root, file)
                    zip_file.write(file_path, os.path.join(dir_name, os.path.relpath(file_path, safe_path)))
        zip_buffer.seek(0)
        return send_file(zip_buffer, mimetype='application/zip', as_attachment=True, download_name=os.path.basename(f"{safe_path}.zip"))
    elif os.path.isfile(safe_path):
        return send_from_directory(loader.doc_path, subpath)
    else:
        abort(404)

@app.get("/api/menu-items")
@token_required
def get_menu_items():
    return loader.get_menu_items()

@app.get("/api/take-chapters")
@token_required
def get_menu_chapter():
    menu = request.args.get('menu')
    return  loader.get_menus_chapter(menu)

@app.get("/api/menu-chapters")
@token_required
def get_md_suites():
    return {'data': loader.get_md_suites()}

@app.get("/api/chapter-mds")
@token_required
def get_chapter_mds():
    menu = request.args.get('menu')
    chapter = request.args.get('chapter')
    return loader.get_chapter_mds(menu, chapter)

@app.get("/api/search-md")
@token_required
def search_markdown_key():
    keyword = request.args.get('keyword')
    return loader.search_keyword(keyword)



# if __name__ == '__main__':
#     app.run(host='0.0.0.0', port=5005, debug=False)