#!/usr/bin/env python
# encoding: utf-8
"""
@author: simin. Gong
@license: (C) Copyright 2013-2023 Spacemit Corporation Limited.
@contact: simin.gong@spacemit.com
@file: loader.py
@date: 2024/12/13 11:27 
@desc:
"""
import base64
import os
import re
import sys
from dataclasses import dataclass, asdict
import pickle
import markdown


@dataclass
class ChapterNode:
    title: str
    icon: str = None
    desc: str = ''
    md_lists: list = None

@dataclass
class MdNode:
    title: str
    content: str

def image_to_base64(image_path):
    with open(image_path, "rb") as image_file:
        image_data = image_file.read()
        base64_string = base64.b64encode(image_data).decode('utf-8')
    return base64_string

def read_file(file_path):
    try:
        print('开始加载md文件：{}'.format(file_path))
        with open(file_path, encoding='utf-8-sig') as f:
            content = f.read()
            return content
    except FileNotFoundError:
        print("文件未找到，请检查文件路径。")
    except Exception as e:
        print(f"读取文件时发生错误: {e}")
    return None


def get_context_from_html(html_content, keyword, context_size=20):
    from bs4 import BeautifulSoup
    soup = BeautifulSoup(html_content, 'lxml')
    text = soup.get_text(separator=' ', strip=True)
    matches = [(m.start(), m.end()) for m in re.finditer(re.escape(keyword), text)]

    contexts = []
    for start, end in matches:
        start_context = max(0, start - context_size)
        end_context = min(len(text), end + context_size)
        context = f"...{text[start_context:start]}[KEYWORD]{text[end:end_context]}..."
        contexts.append(context)
        break
    return contexts

def render_markdown(file_path):
    content = read_file(file_path)
    if content is None:
        return None
    extensions = [  # 根据不同教程加上的扩展
        'markdown.extensions.extra',
        'markdown.extensions.codehilite',  # 代码高亮扩展
        'markdown.extensions.tables',
        'markdown.extensions.fenced_code',
        'mdx_math'
    ]
    data = markdown.markdown(content, extensions=extensions, extension_configs={'mdx_math': { 'enable_dollar_delimiter': True, 'add_preview': True }})

    return data

class MdLoader(object):

    def __init__(self, root_path: str, force_reload: bool = False):
        self.root_path = root_path
        self.doc_path = os.path.join(root_path, 'docs')
        self.code_path = os.path.join(root_path, 'code')
        self.cache_file = os.path.join(root_path, 'cache.json')
        self.md_suites = None
        self.md_content_maps = None
        self.force_reload = force_reload
        self.md_item_suites = []

    def get_root_path(self):
        return self.root_path

    def self_check(self):
        if not os.path.exists(self.doc_path):
            raise Exception('文档路径未找到：{}'.format(self.doc_path))

    def reload(self):
        if self.force_reload:
            self.load()

    def load(self, refresh_cache = False):
        self.self_check()
        if refresh_cache or not self._load_from_cache():
            print('加载缓存文件成功!')
            self.md_suites, self.md_content_maps = self.get_file_structure()
            self._dump_to_cache()
        self.md_item_suites = []
        for key, var in self.md_suites.items():
            self.md_item_suites.append({'title': key, 'items': var})
        print('加载成功')

    def _load_from_cache(self):
        if not os.path.exists(self.cache_file):
            return False
        print('开始加载缓存文件!')
        with open(self.cache_file, 'rb') as file:
            data = pickle.load(file)
            print('使用缓存文件!')
            self.md_suites, self.md_content_maps = data['suites'], data['maps']
        return True

    def _dump_to_cache(self):
        if os.path.exists(self.cache_file):
            os.remove(self.cache_file)
        data = { 'suites': self.md_suites, 'maps': self.md_content_maps }
        with open(self.cache_file, 'wb') as file:
            pickle.dump(data, file)

    def get_md_suites(self):
        self.reload()
        return self.md_item_suites

    def get_menu_items(self):
        self.reload()
        return list(self.md_suites.keys())

    def get_menus_chapter(self, menu: str):
        self.reload()
        if menu not in self.md_suites:
            return []
        return [suite['title'] for suite in self.md_suites[menu]]

    def get_chapter_md(self, url: str):
        self.reload()
        if url not in self.md_content_maps:
            return ''
        return self.md_content_maps[url]

    def get_chapter_mds(self, menu: str, chapter: str):
        self.reload()
        if menu not in self.md_suites:
            return []
        item = self.md_suites[menu]
        for chapter_node in  item:
            if chapter_node['title'] == chapter:
                return chapter_node['md_lists']
        return []

    def search_keyword(self, keyword: str):
        """ 搜索关键字

        :param keyword:
        :return:
        """
        find_urls = []
        for url, content in self.md_content_maps.items():
            if keyword in content['content']:
                find_urls.append(url)
        return find_urls

    def get_file_structure(self):
        """ 遍历所有的文档

        :return:
        """
        file_structure = {}
        md_content_maps = {}
        for item in sorted(os.listdir(self.doc_path)):
            item_path = os.path.join(self.doc_path, item)
            if os.path.isdir(item_path):
                file_structure[item] = []
                for sub_item in sorted(os.listdir(item_path)):
                    sub_item_path = os.path.join(item_path, sub_item)
                    if os.path.isdir(sub_item_path):
                        c_node = ChapterNode(sub_item)
                        logo_path = os.path.join(sub_item_path, 'logo.png')
                        if os.path.exists(logo_path):
                            c_node.icon = f'data:image/png;base64,{image_to_base64(logo_path)}'
                        desc_path = os.path.join(sub_item_path, 'desc.txt')
                        if os.path.exists(desc_path):
                            c_node.desc = read_file(desc_path)
                        md_nodes = []
                        for md_item in os.listdir(sub_item_path):
                            if md_item.endswith('md'):
                                md_nodes.append(md_item.replace('.md', ''))
                                md_content_maps[f'{item}/{sub_item}/{md_item}'] = asdict(MdNode(md_item, render_markdown(os.path.join(sub_item_path, md_item))))
                        c_node.md_lists = md_nodes
                        file_structure[item].append(asdict(c_node))

        return file_structure, md_content_maps